def chains_templates():
    return {
        "snowchat_chain_template": """Considering the provided chat history and a subsequent question, rewrite the follow-up question to be an independent query. Alternatively, conclude the conversation if it appears to be complete.

        Chat History:\"""
        {chat_history}
        \"""
        Follow Up Input: \"""
        {question}
        \"""
        Standalone question:""",
        "snowchat_prompt_template": """ You're a senior SQL developer. You have to write sql code in snowflake database based on the following question. Also you have to ignore the sql keywords and give a one or two sentences about how did you arrive at that sql code. display the sql code in the code format (do not assume anything if the column is not available then say it is not available, do not make up code). Make sure the SQL code you create is a valid SQL ANSI code that works with pyspark dataframes

        Question: {question}
        {context}
        Answer:""",
        "code_extraction_prompt": "Extract the input's text SQL query \n\n{text} \n\n. Only return the SQL code.",
        "validation_prompt": "You're a senior SQL and Machine Learning developer. Review the results provided and return feedback on the code and the answer:",
    }



def agents_templates():
    return {
        "smart_gpt": "You are a helpful assistant that that can answer questions in an a step by step way, making sure to have the right answer. Let's work out the following problem: {prompt}. Now, as a researcher, you are tasked with investigating the provided response options, list the flaws and faulty logic, as well as the correct statements of each answer option. Let's work out step in a step by step way to be sure we have all the errors and correct statements. Then, after discussing the reseached options, as a resolver, you are tasked with 1) finding which of the answer the reseacher though of was best, 2) improving that answer, and 3) returning the improved answer in full. Let's work this out in a step by step way to be sure we have the right answer. At the end, return a user friendly answer as per the initial question following the result result of step 3. Display the pyspark/pandas code used to transform and process the data, and show a sample code to plot the resulting prediction plus the initial dataset."
    }



import streamlit as st

import llm_explorer.interfaces.frontend as it
from llm_explorer.agents import pandas_agent

st.text("LLM Explorer.")
st.warning(
    "It uses Langchain Agent for Pandas with OpenAI GPT3.5 API to generate a step by step explanation of the code used to generate a time series prediction"
)

it.display_sidebar()
pandas_agent()



# --- GENERAL SETTINGS ---
PAGE_TITLE: str = "LLM Explorer"
PAGE_ICON: str = "🤖🎯"
LANG_EN: str = "En"
LANG_RU: str = "Ru"
LANG_ES: str = "Es"

import streamlit as st

from .ui import about

st.set_page_config(
    page_title=PAGE_TITLE,
    page_icon=PAGE_ICON,
    layout="centered",
    initial_sidebar_state="auto",
    menu_items={
        "Report a bug": "mailto:carlos@occlusion.solutions",
        "About": about(),
    },
)

from pathlib import Path

from dotenv import find_dotenv, load_dotenv

from .chains import ExplorerConversationChain
from .chat import chat_loop
from .interfaces import frontend as it
from .ui.lang import en, es


def auth():
    import streamlit_authenticator as stauth
    import yaml
    from yaml.loader import SafeLoader

    with open("llm_explorer/auth/access.yaml") as file:
        config = yaml.load(file, Loader=SafeLoader)

    authenticator = stauth.Authenticate(
        config["credentials"],
        config["cookie"]["name"],
        config["cookie"]["key"],
        config["cookie"]["expiry_days"],
        config["preauthorized"],
    )

    name, authentication_status, username = authenticator.login("Login", "main")

    # if st.session_state["authentication_status"]:
    #     authenticator.logout('Logout', 'main')
    #     st.write(f'Welcome *{st.session_state["name"]}*')
    #     st.title('LLM Explorer')
    # elif st.session_state["authentication_status"] == False:
    #     st.error('Username/password is incorrect')
    # elif st.session_state["authentication_status"] == None:
    #     st.warning('Please enter your username and password')

    if authentication_status:
        authenticator.logout("Logout", "main")
        if username in config["preauthorized"]["users"]:
            st.write(f"Welcome *{username}*")
            return True
        else:
            st.error("Username/password is incorrect")
            return False
    elif authentication_status == False:
        st.error("Username/password is incorrect")
        return False
    elif authentication_status is None:
        st.warning("Please enter your username and password")


load_dotenv(find_dotenv())
chain = ExplorerConversationChain()

# --- PATH SETTINGS ---
current_dir: Path = Path(__file__).parent if "__file__" in locals() else Path.cwd()
css_file: Path = "llm_explorer/ui/styles/.css"
assets_dir: Path = current_dir / "assets"
icons_dir: Path = assets_dir / "icons"
img_dir: Path = assets_dir / "img"
tg_svg: Path = icons_dir / "tg.svg"


# --- LOAD CSS ---
with open(css_file) as f:
    st.markdown(f"<style>{f.read()}</style>", unsafe_allow_html=True)

# selected_lang = option_menu(
#     menu_title=None,
#     options=[LANG_EN, LANG_RU, LANG_ES],
#     icons=["globe2", "translate"],
#     menu_icon="cast",
#     default_index=0,
#     orientation="horizontal",
#     styles=HEADER_STYLES
# )

# # Storing The Context
# if "locale" not in st.session_state:
#     st.session_state.locale = en
# if "generated" not in st.session_state:
#     st.session_state.generated = []
# if "past" not in st.session_state:
#     st.session_state.past = []
# if "messages" not in st.session_state:
#     st.session_state.messages = []
# if "user_text" not in st.session_state:
#     st.session_state.user_text = ""
# if "input_kind" not in st.session_state:
#     st.session_state.input_kind = st.session_state.locale.input_kind_1
# if "seed" not in st.session_state:
#     st.session_state.seed = 420# noqa: S311
# if "costs" not in st.session_state:
#     st.session_state.costs = []
# if "total_tokens" not in st.session_state:
#     st.session_state.total_tokens = []


def main() -> None:
    it.display_header()
    it.display_sidebar()
    if auth():
        chat_loop(chain)
    else:
        st.write("You are not authorized to access this page.")


if __name__ == "__main__":
    main()



import pandas as pd

# import snowflake.connector
import streamlit as st
from databricks import sql as adb_sql


class ADBDDL:
    """
    ADBDDL class loads DDL files for various tables in a database for Databricks and Delta Lake.
    """

    def __init__(self):
        self.ddl_dict = self.load_ddls()

    @staticmethod
    def load_ddls():
        ddl_files = {
            "telemetry": "llm_explorer/sql/ddl_tse_telemetry.sql",
        }

        ddl_dict = {}
        for table_name, file_name in ddl_files.items():
            with open(file_name, "r") as f:
                ddl_dict[table_name] = f.read()
        return ddl_dict

    def query_lakehouse(self, query) -> pd.DataFrame:
        """
        Executes databricks sql query and returns result as data as dataframe.
        Example of parameters
        :param sql: sql query to be executed
        """
        print(f"Received query: {query}")
        try:
            with adb_sql.connect(
                server_hostname=st.secrets.connections.databricks.server_hostname,
                http_path=st.secrets.connections.databricks.http_path,
                access_token=st.secrets.connections.databricks.access_token,
            ) as adb_connection:
                try:
                    with adb_connection.cursor() as cursor:
                        print(f"Opened cursor {cursor}")
                        cursor.execute(query)
                        column_names = [desc[0] for desc in cursor.description]
                        data = cursor.fetchall()
                        print(data)
                        df = pd.DataFrame(data, columns=column_names)
                        return df
                except Exception as e:
                    print(f"Error in cursor {e}")
        except Exception as e:
            print(f"Error in connection {e}")



import re

import openai
import pandas as pd
import streamlit as st
from langchain.chains import ConversationalRetrievalChain, LLMChain, RetrievalQA
from langchain.chains.question_answering import load_qa_chain
from langchain.embeddings.openai import OpenAIEmbeddings
from langchain.prompts.prompt import PromptTemplate

from llm_explorer.llm import set_llm

set_llm()

from llm_explorer.templates.chains import chains_templates


def set_chain(llm, **kwargs):
    condense_question_prompt = PromptTemplate.from_template(
        chains_templates()["snowchat_chain_template"]
    )
    QA_PROMPT = PromptTemplate(
        template=chains_templates()["snowchat_prompt_template"],
        input_variables=["question", "context"],
    )

    question_generator = LLMChain(llm=llm, prompt=condense_question_prompt)

    doc_chain = load_qa_chain(
        llm=llm,
        chain_type=kwargs.get(
            "chain_type", "stuff"
        ),  # Should be one of "stuff","map_reduce", "map_rerank", and "refine".
        prompt=QA_PROMPT,
    )
    return question_generator, doc_chain


class ExplorerConversationChain:
    def __init__(self, **kwargs):
        self.kwargs = kwargs
        self.llm = st.session_state.llm
        self.embeddings = OpenAIEmbeddings(
            openai_api_key=st.secrets.connections.openai.api_key
        )

    def get_qa_retrieval_chain(self, vectorstore):
        return RetrievalQA.from_chain_type(
            llm=self.llm,
            chain_type=self.kwargs.get("chain_type", "stuff"),
            retriever=vectorstore.as_retriever(),
        )

    def get_chat(self, vectorstore, memory):
        question_generator, doc_chain = set_chain(self.llm)
        return ConversationalRetrievalChain(
            retriever=vectorstore.as_retriever(search_kwargs={"k": 3}),
            memory=memory,
            combine_docs_chain=doc_chain,
            question_generator=question_generator,
        )

    # def load_state(self):
    #     memory = ConversationBufferMemory(
    #         memory_key="chat_history", return_messages=True
    #     )
    #     return ConversationalRetrievalChain.from_llm(
    #         OpenAI(temperature=0.8),
    #         self.vectorstore.as_retriever(search_kwargs={"k": 3}),
    #         memory=memory,
    #     )

    def run(self, prompt, vectorstore, memory):
        chain = self.get_chat(vectorstore, memory)
        return chain.run(prompt)


# can be removed with better prompt
def extract_code(text) -> str:
    # from langchain.chat_models import ChatOpenAI
    """
    This function is used to extract the SQL code from the user's input.

    Parameters:
    text (str): The text to be processed.

    Returns:
    str: The SQL code extracted from the user's input.
    """
    if len(text) < 5:
        return None
    # Use OpenAI's GPT-3.5 to extract the SQL code
    response = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=[
            {
                "role": "user",
                "content": chains_templates()["code_extraction_prompt"].format(
                    text=text
                ),
            },
        ],
        # stream=True
    )

    # Extract the SQL code from the response
    sql_code = response.choices[0].message.content

    return sql_code


def is_sql_query(text: str) -> bool:
    """
    Checks if the input text is likely an SQL query.

    :param text: input text
    :return: True if the input is likely an SQL query, False otherwise
    """
    # Define a list of common SQL keywords
    keywords = [
        "SELECT",
        "FROM",
        "WHERE",
        "UPDATE",
        "INSERT",
        "DELETE",
        "JOIN",
        "GROUP BY",
        "ORDER BY",
        "HAVING",
        "LIMIT",
        "OFFSET",
        "UNION",
        "CREATE",
        "ALTER",
        "DROP",
        "TRUNCATE",
        "EXPLAIN",
        "WITH",
    ]

    # Create a single regular expression pattern to search for all keywords
    pattern = r"\b(?:" + "|".join(keywords) + r")\b"

    # Check if any of the keywords are present in the input text (case-insensitive)
    if re.search(pattern, text, re.IGNORECASE):
        return True

    return False



import contextlib

import pandas as pd
import streamlit as st
from dotenv import find_dotenv, load_dotenv
from langchain import ConversationChain, LLMChain, OpenAI
from langchain.agents import (
    AgentExecutor,
    AgentType,
    Tool,
    ZeroShotAgent,
    create_pandas_dataframe_agent,
    initialize_agent,
    load_tools,
)
from langchain.agents.load_tools import get_all_tool_names

import llm_explorer.interfaces.frontend as it
from llm_explorer.templates.agents import agents_templates
from llm_explorer.templates.prompt import prompt_templates

load_dotenv(find_dotenv())


class TSELLMAgent:
    def __init__(self, agent_id: str = "smart_gpt"):
        self.agent_templates = agents_templates()
        self.agent_id = agent_id

    def execute(self, query, df):
        with contextlib.suppress(Exception):
            # Try to get the ts_id from the dataframe
            ts_id = df["ts_id"].unique()[0]
            df.drop(columns=["ts_id"], inplace=True)
            cols = df.columns.tolist()
            return self.run_pandas_agent(
                query.format({"ts_id": ts_id, "cols": cols}), df.tail(10)
            )
        return self.run_pandas_agent(query, df.tail(10))

    def run_pandas_agent(
        self,
        query: str,
        df,
    ) -> str:
        llm = OpenAI()
        get_all_tool_names()
        tools = load_tools(
            ["llm-math", "open-meteo-api", "requests_all", "terminal", "python_repl"],
            llm=llm,
        )

        agent = create_pandas_dataframe_agent(
            llm=llm,
            df=df,
            tools=tools,
            # verbose=True,
            max_iterations=15,
            early_stopping_method="force",
        )
        return agent.run(self.agent_templates[self.agent_id].format(prompt=query))

    def run_agent_query(self, uploaded_file, query):
        st.write("Executing LLM Agent...")
        try:
            df = pd.read_csv(uploaded_file)
            st.dataframe(df.style.highlight_max(axis=0))
        except Exception as e:
            print(e)
            df = None
        with st.spinner(text="Executing LLM Agent..."):
            agent_results = self.execute(query=query, df=df)
            st.success("Completed inferece!")
            st.markdown(f"**Explanation:** {agent_results}")


def pandas_agent():
    uploaded_file = it.display_widgets(
        "llm_explorer/indexes/samples/telemetry_sample_forecast.csv"
    )
    sample_df = pd.read_csv(uploaded_file)
    st.write("Sample of loaded table:")
    st.table(sample_df.head(3))
    # Create the Streamlit dropdown
    option = st.selectbox(
        "Sample Prompt",
        list(prompt_templates().keys()),
    )
    query = ""
    if not (query):
        st.error("Edit Inserted Prompt")
    query = st.text_area(
        label=f"{option}:",
        value=prompt_templates()[option],
        height=200,
    )

    agent = TSELLMAgent()
    if st.button("Run"):
        agent.run_agent_query(uploaded_file=uploaded_file, query=query)


from typing import Type

import streamlit as st
from transformers import HfAgent, OpenAiAgent

# import soundfile as sf

# def play_audio(audio):
#     sf.write("speech_converted.wav", audio.numpy(), samplerate=16000)


class TransformerAgents:
    @staticmethod
    def load_starcoder_agent(
        api_path: str = "https://api-inference.huggingface.co/models/bigcode/starcoder",
    ):
        return HfAgent(api_path)

    @staticmethod
    def load_openassistant_agent(
        api_path: str = "https://api-inference.huggingface.co/models/OpenAssistant/oasst-sft-4-pythia-12b-epoch-3.5",
    ):
        return HfAgent(api_path)

    @staticmethod
    def load_openai_agent(
        model: str = "gpt-3.5-turbo",
        api_key: str = st.secrets.connections.openai.api_key,
    ):
        return OpenAiAgent(model=model, api_key=api_key)


class HFTAgent:
    def __init__(self, agent_name: str = "openai", **kwargs):
        def get_agent_strategy(agent_name):
            agent_strategies = {
                "openai": TransformerAgents.load_openai_agent,
                "starcoder": TransformerAgents.load_starcoder_agent,
                "openassistant": TransformerAgents.load_openassistant_agent,
            }
            return agent_strategies[agent_name]()

        self.kwargs = kwargs
        self.agent = get_agent_strategy(agent_name)

    def run(self, query):
        return self.agent.run(query, **self.kwargs)

    def chat(self, query):
        return self.agent.chat(query, **self.kwargs)

    def clean_chat(self):
        self.agent.prepare_for_new_chat()


from typing import List

import pandas as pd
import streamlit as st
from langchain.agents import AgentExecutor, create_sql_agent, load_tools
from langchain.agents.agent_toolkits.base import BaseToolkit
from langchain.agents.load_tools import get_all_tool_names
from langchain.base_language import BaseLanguageModel
from langchain.llms.openai import OpenAI
from langchain.memory import ConversationBufferMemory
from langchain.sql_database import SQLDatabase
from langchain.tools import BaseTool
from langchain.tools.sql_database.tool import (
    InfoSQLDatabaseTool,
    ListSQLDatabaseTool,
    QueryCheckerTool,
    QuerySQLDataBaseTool,
)
from pydantic import Field
from sqlalchemy import *
from sqlalchemy import MetaData, Table, create_engine, select
from sqlalchemy.engine import create_engine
from sqlalchemy.schema import *

# from pip._internal import main as pipmain
# pipmain(['install', 'sqlalchemy-databricks'])


class LLMExplorerToolkit(BaseToolkit):
    """
    Toolkit for LLM Explorer, lakehouse interaction.

    This is a subclass of the BaseToolkit class from the langchain.agents.agent_toolkits.base module.
    It defines a set of tools that can be used to interact with a SQL database.
    """

    db: SQLDatabase = Field(exclude=True)
    llm: BaseLanguageModel = Field(exclude=True)

    @property
    def dialect(self) -> str:
        """
        Return string representation of dialect to use.
        """
        return self.db.dialect

    class Config:
        """
        Configuration for this Pydantic object.
        """

        arbitrary_types_allowed = True

    def get_tools(self) -> List[BaseTool]:
        """
        Get the tools in the toolkit.
        """
        return [
            QuerySQLDataBaseTool(db=self.db),
            InfoSQLDatabaseTool(db=self.db),
            ListSQLDatabaseTool(db=self.db),
            QueryCheckerTool(db=self.db, llm=self.llm),
        ] + self.define_explorer_tools()

    def define_explorer_tools(self):
        """
        Define the LLM Explorer tools to be used in the toolkit.
        """
        get_all_tool_names()
        return load_tools(
            ["llm-math", "open-meteo-api", "requests_all", "terminal", "python_repl"],
            llm=self.llm,
        )


class ExplorerAgent:
    """
    An agent designed to interact with a SQL database and return answers to user input questions.

    Attributes:
    -----------
    _server_hostname: str
        The hostname of the Databricks server.
    _http_path: str
        The path to access the Databricks server over HTTP.
    _access_token: str
        The access token used to authenticate the agent with the Databricks server.
    _port: int
        The port number to connect to the Databricks server.
    _database: str
        The name of the database to connect to.
    _engine: sqlalchemy.engine.base.Engine
        The database engine used to connect to the Databricks server.
    _llm: openai_sql.OpenAI
        The OpenAI language model used to generate SQL queries.
    _database_connection: openai_sql.SQLDatabase
        The connection to the SQL database.
    _toolkit: openai_sql.LLMExplorerToolkit
        The toolkit used to explore the SQL database.
    memory: openai_sql.ConversationBufferMemory
        The conversation memory buffer used to store previous user inputs and outputs.
    SQL_PREFIX: str
        The prefix added to the SQL query generated by the agent.
    SQL_SUFFIX: str
        The suffix added to the SQL query generated by the agent.
    PREFIX: str
        The prefix added to the input question asked by the agent.
    FORMAT_INSTRUCTIONS: str
        The format instructions for the user's response.
    agent_executor: openai_sql.SQLAgent
        The SQL agent that executes the SQL query generated by the agent.

    Methods:
    --------
    __init__(self, **kwargs):
        Initializes the ExplorerAgent class by setting its attributes and creating the necessary connections.
    """

    def __init__(self, **kwargs):
        """
        Initializes the ExplorerAgent class by setting its attributes and creating the necessary connections.

        Parameters:
        -----------
        **kwargs: dict
            A dictionary of keyword arguments to set optional attributes.
        """
        # Set attributes from Streamlit secrets and input parameters
        self._server_hostname = st.secrets.connections.databricks.server_hostname
        self._http_path = st.secrets.connections.databricks.http_path
        self._access_token = st.secrets.connections.databricks.access_token
        self._port = 443
        self._database = kwargs.get("database", "default")

        # Create the database engine and OpenAI language model
        self._engine = create_engine(
            f"databricks+connector://token:{self._access_token}@{self._server_hostname}:{self._port}/{self._database}",
            connect_args={
                "http_path": self._http_path,
            },
        )
        self._llm = OpenAI(temperature=0, verbose=True)
        self._database_connection = SQLDatabase(engine=self._engine)
        self._toolkit = LLMExplorerToolkit(
            db=self._database_connection, llm=self._llm, verbose=True
        )
        self.memory = ConversationBufferMemory(memory_key="chat_history")

        self.SQL_PREFIX = """You are an agent designed to leverage the provided tools and datasets to answer at the best of your ability, question related to SQL databases focused on pyspark sql, but can handle any dialect.
        As an agent, you can solve both general questions that may or may not require access to the database, and specific questions that require access to the database, tools and external services. In any case, your goal is to answer the question as best as you can. 
        When you find out that you need to run SQL queries to create an answer, given an input question, create a syntactically correct {dialect} query to run, then look at the results of the query and return the answer.
        Unless the user specifies a specific number of examples they wish to obtain, always limit your query to at most {top_k} results.
        You can order the results by a relevant column to return the most interesting examples in the database.
        Never query for all the columns from a specific table, only ask for the relevant columns given the question. 
        When using pyspark/databricks sql dialtect, do not use double quotes to build your query's columns names -only if you are using a different dialect, or it is extremely necessary-.
        You have access to tools for interacting with the database.
        You MUST double check your query before executing it. If you get an error while executing a query, rewrite the query and try again.
        You may use tools to run python or pyspark scripts that use the dataframes returned by the SQL queries.
        If your prompt is too long, you can break up the data in chunks and run the first analysist using the chunk with the most recent data.
        Avoid creating queries that require too many tokens to read, so always include filters and limits.
        Make sure the you dont pass the 4097 tokens limit

        DO NOT make any DML statements (INSERT, UPDATE, DELETE, DROP etc.) to the database.

        Return the results in a string formatting the code or dataframe in a way that is easy to read.
        """

        self.SQL_SUFFIX = """Begin!

        Question: {input}
        Thought: I should look at the tables in the database to see what I can query.
        {agent_scratchpad}"""

        self.FORMAT_INSTRUCTIONS = """Use the following format:

        Question: the input question you must answer
        Thought: you should always think about what to do
        Action: the action to take, should be one of [{tool_names}]
        Action Input: the input to the action
        Observation: the result of the action
        ... (this Thought/Action/Action Input/Observation can repeat N times)
        Thought: I now know the final answer
        Final Answer: the final answer to the original input question."""
        SUFFIX = """Begin!

        Question: {input}
        Thought:{agent_scratchpad}"""

        self.agent_executor = create_sql_agent(
            llm=self._llm,
            toolkit=self._toolkit,
            verbose=True,
            prefix=self.SQL_PREFIX,
            suffix=self.SQL_SUFFIX,
            format_instructions=self.FORMAT_INSTRUCTIONS,
            top_k=10,
            max_iterations=15,
            max_execution_time=None,
            early_stopping_method="force",
        )

    def run(self, query):
        return self.agent_executor.run(query)



import streamlit as st
from streamlit import components


class UI:
    def __init__():
        ...

    def chat_md():
        st.markdown(
            '<div id="input-container-placeholder"></div>', unsafe_allow_html=True
        )
        components.v1.html(
            """
            <script>
            window.addEventListener('load', function() {
                const inputContainer = document.querySelector('.stTextInput');
                const inputContainerPlaceholder = document.getElementById('input-container-placeholder');
                inputContainer.id = 'input-container';
                inputContainerPlaceholder.appendChild(inputContainer);
                document.getElementById("input").focus();
            });
            </script>
            """,
            height=0,
        )

    def message_func(text, is_user=False):
        """
        This function is used to display the messages in the chatbot UI.

        Parameters:
        text (str): The text to be displayed.
        is_user (bool): Whether the message is from the user or the chatbot.
        key (str): The key to be used for the message.
        avatar_style (str): The style of the avatar to be used.
        """
        if is_user:
            avatar_url = "https://avataaars.io/?avatarStyle=Transparent&topType=ShortHairShortFlat&hairColor=Black&facialHairType=BeardLight&facialHairColor=Black&clotheType=Hoodie&clotheColor=DarkBlue&eyeType=Squint&eyebrowType=DefaultNatural&mouthType=Smile&skinColor=Light"
            message_alignment = "flex-end"
            message_bg_color = "linear-gradient(135deg, #00B2FF 0%, #006AFF 100%)"
            avatar_class = "user-avatar"
            st.write(
                f"""
                    <div style="display: flex; align-items: center; margin-bottom: 10px; justify-content: {message_alignment};">
                        <div style="background: {message_bg_color}; color: white; border-radius: 20px; padding: 10px; margin-right: 5px; max-width: 75%;">
                            {text}
                        </div>
                                <img src="{avatar_url}" class="{avatar_class}" alt="avatar" />

                    </div>
                    """,
                unsafe_allow_html=True,
            )
        else:
            avatar_url = "https://avataaars.io/?avatarStyle=Transparent&topType=Hat&accessoriesType=Prescription02&facialHairType=BeardMajestic&facialHairColor=Platinum&clotheType=BlazerShirt&eyeType=Close&eyebrowType=SadConcerned&mouthType=Twinkle&skinColor=Brown"
            message_alignment = "flex-start"
            message_bg_color = "#71797E"
            avatar_class = "bot-avatar"
            st.write(
                f"""
                    <div style="display: flex; align-items: center; margin-bottom: 10px; justify-content: {message_alignment};">
                        <img src="{avatar_url}" class="{avatar_class}" alt="avatar" />
                        <div style="background: {message_bg_color}; color: white; border-radius: 20px; padding: 10px; margin-right: 5px; max-width: 75%;">
                            {text} \n </div>
                    </div>
                    """,
                unsafe_allow_html=True,
            )

    def styles():
        return """
<link rel="preconnect" href="https://fonts.gstatic.com">
<link href="https://fonts.googleapis.com/css2?family=Roboto+Slab:wght@700&display=swap" rel="stylesheet">
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.1.0/css/all.min.css">

<style>
    #input-container {
        position: fixed;
        bottom: 0;
        width: 100%;
        padding: 10px;
        background-color: white;
        z-index: 100;
    }
    h1 {
        font-family: 'Roboto Slab', serif;
    }
    .user-avatar {
        float: right;
        width: 40px;
        height: 40px;
        margin-left: 5px;
        margin-bottom: -10px;
        border-radius: 50%;
        object-fit: cover;
    }
    .bot-avatar {
        float: left;
        width: 40px;
        height: 40px;
        margin-right: 5px;
        border-radius: 50%;
        object-fit: cover;
    }
</style>
"""

    def sidebar():
        return """
# LLM Explorer

## Features

- **Natural Language Processing**: Understands your text queries and converts them into SQL and python queries
- **Instant Results**: Fetches data from Engines and displays the results quickly. Create Job Manifest for Orchestrated pipelines deployments
- **GEN AI models**: Uses OpenAI's GPT-4, langchain and vector search (into loaded documents for databases metadata & codebase) to generate queries for the lakehouse

Here are some example queries you can try with LLM Explorer:

- Forecast the next the 3 days of data from the time series in your memory
- What are the top 10 producing wells?
- What is the site with most wells and what are their production?

Streamlit is an open-source app framework built specifically for
Machine Learning and Data Science projects.
### Want to learn more?
- Check out [streamlit.io](https://streamlit.io)
- Jump into our [documentation](https://docs.streamlit.io)
- Ask a question in our [community
    forums](https://discuss.streamlit.io)
### See more complex demos
- Use a neural net to [analyze the Udacity Self-driving Car Image
    Dataset](https://github.com/streamlit/demo-self-driving)
- Explore a [New York City rideshare dataset](https://github.com/streamlit/demo-uber-nyc-pickups)
"""


def about():
    return """
# LLM Base
***
This repository includes two initial execution modes. The python notebook is a direct implementation of the LLM agent, while the tse_llm_explorer is a python script to be executed with streamlit (streamit run tse_llm_explorer.py). The streamlit app is a simple interface to interact with the LLM agent.
***

This repository focuses on experimenting with the LangChain library for building powerful applications with large language models (LLMs). By leveraging state-of-the-art language models like OpenAI's GPT-3.5 Turbo (and soon GPT-4).

LangChain is a comprehensive framework designed for developing applications powered by language models. It goes beyond merely calling an LLM via an API, as the most advanced and differentiated applications are also data-aware and agnostic, enabling language models to connect with other data sources and interact with their environment. The LangChain framework is specifically built to address these principles.

## LangChain

The Python-specific portion of LangChain's documentation covers several main modules, each providing examples, how-to guides, reference docs, and conceptual guides. These modules include:

1. Models: Various model types and model integrations supported by LangChain.
3. Prompts: Prompt management, optimization, and serialization.
3. Memory: State persistence between chain or agent calls, including a standard memory interface, memory implementations, and examples of chains and agents utilizing memory.
4. Indexes: Combining LLMs with custom text data to enhance their capabilities.
5. Chains: Sequences of calls, either to an LLM or a different utility, with a standard interface, integrations, and end-to-end chain examples.
6. Agents: LLMs that make decisions about actions, observe the results, and repeat the process until completion, with a standard interface, agent selection, and end-to-end agent examples.

## Use Cases
With LangChain, developers can create various applications, such as customer support chatbots, automated content generators, data analysis tools, and intelligent search engines. These applications can help businesses streamline their workflows, reduce manual labor, and improve customer experiences.

## Service
By selling LangChain-based applications as a service to businesses, you can provide tailored solutions to meet their specific needs. For instance, companies can benefit from customizable chatbots that handle customer inquiries, personalized content creation tools for marketing, or internal data analysis systems that harness the power of LLMs to extract valuable insights. The possibilities are vast, and LangChain's flexible framework makes it the ideal choice for developing and deploying advanced language model applications in diverse industries.

## Requirements

- [Python 3.6 or higher](https://www.python.org/downloads/)
- [LangChain library](https://python.langchain.com/en/latest/index.html)
- [OpenAI API key](https://platform.openai.com/)
- [SerpAPI API Key](https://serpapi.com/)

## OpenAI API Models
The OpenAI API is powered by a diverse set of [models](https://platform.openai.com/docs/models) with different capabilities and price points. You can also make limited customizations to our original base models for your specific use case with fine-tuning.

## Installation

#### 1. Create a Python environment

Python 3.6 or higher using `venv` or `conda`. Using `venv`:

``` bash
cd tse-llm
python3 -m venv env
source env/bin/activate
```

Using `conda`:
``` bash
cd tse-llm
conda create -n tse-llm-env python=3.11
conda activate tse-llm-env
```

#### 3. Install the required dependencies
``` bash
pip install -r requirements.txt
```

#### 4. Set up the keys in a .env file

First, create a `.env` file in the root directory of the project. Inside the file, add your OpenAI API key:

```makefile
OPENAI_API_KEY=your_api_key_here
```

Save the file and close it. In your Python script or Jupyter notebook, load the `.env` file using the following code:
```python
from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())
```

By using the right naming convention for the environment variable, you don't have to manually store the key in a separate variable and pass it to the function. The library or package that requires the API key will automatically recognize the `OPENAI_API_KEY` environment variable and use its value.

When needed, you can access the `OPENAI_API_KEY` as an environment variable:
```python
import os
api_key = os.environ['OPENAI_API_KEY']
```

Now your Python environment is set up, and you can proceed with running the experiments.

## Run Streamlit App

You can run the app by running:

```bash
streamlit run explorer.py
```
            """



def prompt_templates():
    return {
        "Simple Forecast Prompt": "Perform a basic analysis of the dataset stored in your memory and provide a forecast for the next 1, 7 15 and 30 days. Present the results in markdown. Use your tools as needed to complete the request. When you need to run a pip install command, use the terminal tool instead",
        "Time Series Prediction Prompt with Prophet Model and Ensemble Smoothing": "Use the information stored in your memory to describe the dynamics of the time series behavior and create the prediction for the next 7 days and return the results in markdown string. The dataframe frame includes the columns: {cols}, where ts_id is an identifier for this series with code: {ts_id}. ds is the date, y is the value of the time series, Prediction values can be included or could be calculated by you such that prediction is the predicted value, lower_bound is the lower bound of the prediction, upper_bound is the upper bound of the prediction, abs_error is the absolute error of the prediction. The prediction values are created by a custom prophet model, use them for feature engineering with an ensemble model with TFT and NHITs in the pytorch forecasting library to further smooth the prediction. In order to run pip install or other shell related calls, use the % prefix to run shell commands. When running the data processing jobs, keep a state of the dataframe so that it can be reused in the next step. Only show the specific code used to generate the prediction at the end of the processes when formating the final answer",
        "Time Series Prediction Simplified Prompt": "Generate a markdown string with predicted values for the next 7 days of a time series identified by {ts_id}. Use a custom Prophet model to generate the predictions and smooth them using an ensemble model with TFT and NHITs in the PyTorch forecasting library. You can run shell commands with the % prefix and reuse the dataframe in subsequent steps. The predicted values should include 'prediction', 'lower_bound', 'upper_bound', and 'abs_error'. Only include the code used to generate the prediction in the final answer.",
    }



import streamlit as st
from langchain.llms import OpenAI


def set_llm(**kwargs):
    st.session_state.llm = OpenAI(
        temperature=kwargs.get("temperature", 0),
        openai_api_key=st.secrets.connections.openai.api_key,
        model_name="gpt-3.5-turbo",
        max_tokens=kwargs.get("max_tokens", -1),  # Max number of tokens to generate
        # top_p=kwargs.get(
        #     "top_p", 1
        # ),  # Total probability mass of tokens to consider at each step
        # frequency_penalty=kwargs.get(
        #     "frequency_penalty", 0
        # ),  # Penalizes repeated tokens according to frequency
        # presence_penalty=kwargs.get(
        #     "presence_penalty", 0
        # ),  # Penalizes repeated tokens.
        # n=kwargs.get("n", 1),  # How many completions to generate for each prompt.
        # best_of=kwargs.get(
        #     "best_of", 1
        # ),  # Generates best_of completions server-side and returns the "best".
    )



import pandas as pd
import streamlit as st
from langchain.document_loaders import DataFrameLoader, UnstructuredMarkdownLoader
from langchain.embeddings import OpenAIEmbeddings
from langchain.embeddings.openai import OpenAIEmbeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain.vectorstores import FAISS, Chroma

embeddings = OpenAIEmbeddings(openai_api_key=st.secrets.connections.openai.api_key)


class VectorStoreHandler:
    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def get_vectorstore(self):
        get_vectorstore_strategies = {
            "chroma": load_chroma_vectorstore,
            "faiss": load_faiss_vectorstore,
        }
        vectorstore_strategy = self.kwargs.get("vectorstore", "chroma")
        return get_vectorstore_strategies[vectorstore_strategy]()

    def set_vectorstore(self):
        set_vectorstore_strategies = {
            "chroma": pandas_df_vectorstore_loader,
            "faiss": faiss_metadata_index_loader,
        }
        vectorstore_strategy = self.kwargs.get("vectorstore", "chroma")
        return set_vectorstore_strategies[vectorstore_strategy]()


@st.cache_resource
def load_chroma_vectorstore():
    return Chroma(persist_directory="croma_index", embedding_function=embeddings)


@st.cache_resource
def load_faiss_vectorstore():
    return FAISS.load_local("indexes/faiss_index", embeddings)


def faiss_metadata_index_loader(
    metadata_path: str = "llm_explorer/indexes/metadata/schema.md",
    page_content_column: str = "y",
):
    loader = UnstructuredMarkdownLoader(metadata_path)
    data = loader.load()
    # df = pd.read_csv(data_path)
    text_splitter = RecursiveCharacterTextSplitter(chunk_size=1000, chunk_overlap=20)
    texts = text_splitter.split_documents(data)

    # df_loader = DataFrameLoader(df, page_content_column=page_content_column)
    # docs = df_loader.load()

    faiss_store = FAISS.from_documents(texts, embeddings)
    # docsearch.add_documents(docs)
    faiss_store.save_local("llm_explorer/indexes/faiss_index")

    # with open("vectors.pkl", "wb") as f:
    #     pickle.dump(docsearch, f)


def pandas_df_vectorstore_loader(
    data_path: str = "llm_explorer/indexes/samples/telemetry_sample_forecast.csv",
    page_content_column: str = "y",
):
    df = pd.read_csv(data_path)
    # jdf = df.to_dict(orient='split')
    loader = DataFrameLoader(df, page_content_column=page_content_column)
    docs = loader.load()

    # VectorStoreRetrieverMemory

    vectorstore_ts = Chroma.from_documents(
        docs, embeddings, persist_directory="croma_index"
    )
    # docs = pandas_df_vectorstore_loader(data_path=df_path,  page_content_column=data_columnn)
    vectorstore_ts.persist()

    return docs



# import pyautogui
import streamlit as st
from streamlit.runtime.uploaded_file_manager import UploadedFile

from llm_explorer.databases.adbddl import ADBDDL
from llm_explorer.indexes.vectorstore import faiss_metadata_index_loader
from llm_explorer.ui import UI

adb_ddl = ADBDDL()


def display_header() -> None:
    # st.image("img/logo.jpg")
    st.title("LLM Explorer")
    # Create a description based on the smart gpt and queried defined here
    st.caption(
        "Self service Tool for Interaction, jobs creation, interaction with Engines and Lakehouse Analysis"
    )
    styles_content = UI.styles()
    st.write(styles_content, unsafe_allow_html=True)


def display_widgets(default_path):
    file = st.file_uploader("Upload your file here.")
    if not (file):
        st.error("Upload a time series file .csv with columns: ts_id, ds, y")
    return file or default_path


def retrieve_content_from_file(file: UploadedFile) -> str:
    return file.getvalue().decode("utf8")


def display_sidebar() -> None:
    sidebar_content = UI.sidebar()
    st.sidebar.markdown(sidebar_content)

    # Create a sidebar with a dropdown menu
    selected_table = st.sidebar.selectbox(
        "Select a table:", options=list(adb_ddl.ddl_dict.keys())
    )
    st.sidebar.markdown(f"### DDL for {selected_table} table")
    st.sidebar.code(adb_ddl.ddl_dict[selected_table], language="sql")

    # create a button in the side bar that will move to the next page/radio button choice
    next = st.sidebar.button("Next on list")
    ## will use this list and next button to increment page, MUST BE in the SAME order
    # as the list passed to the radio button
    new_choice = [
        "Explorer",
        "Update Vector Store",
    ]
    choice = st.sidebar.radio(
        "Run",
        (
            "Explorer",
            "Update Vector Store",
        ),
    )
    # Take care of scrolling
    if next:
        ...
        # pyautogui.press("tab")
        # pyautogui.press("down")
    elif choice == "Explorer":
        st.write("Explorer")
    elif choice == "Update Vector Store":
        faiss_metadata_index_loader()
        st.write("Run Update Vector Store")
    elif choice == "llm_explorer/pages/pandas_agent":
        st.write("Run pages/pandas_agent")



import re
import sys

from setuptools import find_packages, setup

if sys.version_info < (3, 6):
    print("Error: llm_explorer does not support this version of Python.")
    print("Please upgrade to Python 3.6 or higher.")
    sys.exit(1)

try:
    from setuptools import find_namespace_packages

    find_namespace_packages()
except ImportError:
    # the user has a downlevel version of setuptools.
    print("Error: llm_explorer requires setuptools v40.1.0 or higher.")
    print(
        'Please upgrade setuptools with "pip install --upgrade setuptools" '
        "and try again"
    )
    sys.exit(1)

version = "#{PKG_VAR_SETUP}#"
package_name = "llm_explorer"

package_env = re.sub(r"[^a-zA-Z]", "", version)

if "PKGVARSETUP" in package_env:
    version = "0.0.3"

package_env = re.sub(r"[^a-zA-Z]", "", version)

# Check if any letters were found
if len(package_env) > 0:
    package_name = package_name + f"_{package_env}"


with open("requirements.txt") as f:
    required = f.read().splitlines()

print(package_name, version)
setup(
    name=package_name,
    version=version,
    author="Carlos D. Escobar-Valbuena",
    author_email="carlosdavidescobar@gmail.com",
    description="A Lakehouse LLM Explorer. Wrapper for spark, databricks and langchain processes",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Occlusion-Solutions/occlussion_llm_explorer.git",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "."},
    packages=find_packages(where="."),
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=required,
    package_data={
        "": ["*.json"],
    },
)



import contextlib

import streamlit as st
from langchain.memory import ConversationBufferMemory

from llm_explorer.agents import ExplorerAgent, HFTAgent
from llm_explorer.chains import extract_code, is_sql_query
from llm_explorer.databases.adbddl import ADBDDL
from llm_explorer.indexes.vectorstore import VectorStoreHandler
from llm_explorer.llm import set_llm
from llm_explorer.templates.chains import chains_templates
from llm_explorer.templates.ui import session_state_templates
from llm_explorer.ui import UI

set_llm()

adb_ddl = ADBDDL()
agent_database = (
    st.session_state["agent_database"]
    if "agent_database" in st.session_state
    else "default"
)
explorer_agent = ExplorerAgent(**{"database": agent_database})
hft_agent = HFTAgent()
faiss_memory = ConversationBufferMemory(memory_key="chat_history", return_messages=True)


def update_progress_bar(value, prefix, progress_bar=None):
    if progress_bar is None:
        progress_bar = st.empty()

    key = f"{prefix}_progress_bar_value"
    if key not in st.session_state:
        st.session_state[key] = 0

    st.session_state[key] = value
    progress_bar.progress(st.session_state[key])
    if value == 100:
        st.session_state[key] = 0
        progress_bar.empty()


def set_chat_state():
    if "generated" not in st.session_state:
        st.session_state["generated"] = [session_state_templates()["greeting"]]
    if "past" not in st.session_state:
        st.session_state["past"] = session_state_templates()["prior"]
    if "input" not in st.session_state:
        st.session_state["input"] = session_state_templates()["starting_query"]
    if "stored_session" not in st.session_state:
        st.session_state["stored_session"] = []
    if "messages" not in st.session_state:
        st.session_state["messages"] = [session_state_templates()["default_messages"]]
    if "query_count" not in st.session_state:
        st.session_state["query_count"] = 0
    if "session_dataframe" not in st.session_state:
        st.session_state["session_dataframe"] = None

    chat_history = session_state_templates()["prior"]
    MAX_INPUTS = 3

    return chat_history, MAX_INPUTS


def validate_response(response, chain, chat_history, vectorstore, enabled=False):
    if enabled:
        # # Faiss Chain Validation
        validation_faiss_memory = ConversationBufferMemory(
            memory_key="chat_history", return_messages=True
        )
        validation_query = f"{chains_templates()['validation_prompt']} \n {response}"
        return chain.run(
            {"question": validation_query, "chat_history": chat_history},
            vectorstore,
            validation_faiss_memory,
        )
    else:
        return response


def run_chat_chain(chain, query, chat_history):
    with st.spinner(text="Executing LLM Chain..."):
        submit_progress_bar = st.empty()
        messages = st.session_state["messages"]

        # # Chroma Chain
        # chroma_memory = ConversationBufferMemory(memory_key="chat_history", return_messages=True)
        # chroma_vectorstore = VectorStoreHandler(**{"vectorstore":"chroma"}).get_vectorstore()
        # chroma_result = chain.run({"question": query, "chat_history": chat_history}, chroma_vectorstore, chroma_memory)
        # update_progress_bar(22, 'submit', submit_progress_bar)

        # Faiss Chain

        faiss_vectorstore = VectorStoreHandler(
            **{"vectorstore": "faiss"}
        ).get_vectorstore()
        faiss_result = chain.run(
            {"question": query, "chat_history": chat_history},
            faiss_vectorstore,
            faiss_memory,
        )
        update_progress_bar(44, "submit", submit_progress_bar)
        result = validate_response(
            faiss_result, chain, chat_history, faiss_vectorstore, enabled=False
        )
        update_progress_bar(66, "submit", submit_progress_bar)

        chat_history.append(result)
        messages.append((query, result))
        st.session_state.generated.append(result)
        update_progress_bar(88, "submit", submit_progress_bar)

        st.session_state.past.append(query)
        st.session_state["query_count"] += 1
        update_progress_bar(100, "submit", submit_progress_bar)
        return result, messages


def show_chat_buttons() -> None:
    b0, b1 = st.columns(2)
    with b0, b1:
        reset_button = b0.button("Reset Chat History")
        save_button = b1.download_button(
            label="Save Chat History",
            data="\n".join([str(d) for d in st.session_state.messages[1:]]),
            file_name="ai-talks-chat.json",
            mime="application/json",
        )
    return reset_button, save_button


def chat_loop(chain):
    chat_history, MAX_INPUTS = set_chat_state()

    RESET = True
    messages_container = st.container()

    with st.form(key="tse_chat_form"):
        query = st.text_area(
            "Query: ",
            key="input",
            value="",
            placeholder=session_state_templates()["query_form"],
            label_visibility="hidden",
            height=150,
        )
        submit_button = st.form_submit_button(label="Submit")

    reset_button, save_button = show_chat_buttons()

    # col1, col2 = st.columns([1, 3.2])

    if reset_button or st.session_state["query_count"] >= MAX_INPUTS and RESET:
        RESET = False
        st.session_state["query_count"] = 0
        set_chat_state()

    selection = st.select_slider(
        "Select an interaction mode",
        options=["chain", "agent", "chat"],
        value=("agent"),
    )
    st.write("You selected ", selection)

    if selection == "chain":
        ## Call LLM Api and Generate Results
        if len(query) > 2 and submit_button:
            result, messages = run_chat_chain(
                chain=chain, query=query, chat_history=chat_history
            )

    elif selection == "agent":
        ## Call Agent Api and Generate Results
        if len(query) > 2 and submit_button:
            with st.spinner(text="Executing LLM Agent..."):
                result = str(explorer_agent.run(query))
                update_chat_state(query, result)
    elif selection == "chat":
        ## Call Chat Api and Generate Results
        if len(query) > 2 and submit_button:
            with st.spinner(text="Executing LLM Chat..."):
                result = hft_agent.chat(query)
                update_chat_state(query, result)

    with messages_container:
        if st.session_state["generated"]:
            for i in range(len(st.session_state["generated"])):
                prior_query = st.session_state["past"][i]
                result_query = st.session_state["generated"][i]

                UI.message_func(prior_query, is_user=True)
                UI.message_func(result_query)
                try:
                    if i > 0 and is_sql_query(result_query):
                        code, run_query = run_generated_code(result_query)
                except Exception as e:
                    print(e)

    if st.session_state["query_count"] == MAX_INPUTS and RESET:
        st.warning(
            "You have reached the maximum number of inputs. The chat history will be cleared after the next input."
        )

    # col2.markdown(f'<div style="line-height: 2.5;">{st.session_state["query_count"]}/{MAX_INPUTS}</div>', unsafe_allow_html=True)

    UI.chat_md()


def update_chat_state(query, result):
    st.session_state.messages.append((query, result))
    st.session_state.past.append(query)
    st.session_state.generated.append(result)


def run_generated_code(result_query):
    code = extract_code(result_query)
    user_query = st.text_area(
        label=f"Run Query: {code}:", value=code, height=200, label_visibility="hidden"
    )
    run_query = st.button(label=f"Run Query: {user_query}")
    with contextlib.suppress(Exception):
        if code and run_query:
            with st.spinner(text="Running SQL Query..."):
                df = adb_ddl.query_lakehouse(user_query)
                st.session_state["session_dataframe"] = df
                print(df)
                st.dataframe(df, use_container_width=True)
    return code, run_query



from dataclasses import dataclass
from typing import List  # NOQA: UP035

AI_ROLE_OPTIONS_EN = [
    "helpful assistant",
    "code assistant",
    "code reviewer",
    "text improver",
    "cinema expert",
    "sport expert",
    "online games expert",
    "food recipes expert",
    "English grammar expert",
    "friendly and helpful teaching assistant",
    "laconic assistant",
    "helpful, pattern-following assistant",
    "translate corporate jargon into plain English",
]


AI_ROLE_OPTIONS_ES = [
    "asistente útil",
    "asistente de código",
    "revisor de código",
    "mejorador de texto",
    "experto en cine",
    "experto en deportes",
    "experto en juegos en línea",
    "experto en recetas de cocina",
    "experto en gramática inglesa",
    "asistente de enseñanza amable y servicial",
    "asistente lacónico",
    "asistente útil y siguiendo patrones",
    "traductor de jerga corporativa a español claro",
]


REPO_URL: str = "https://github.com/Occlusion-Solutions/occlussion_llm_explorer.git"
README_URL: str = f"{REPO_URL}#readme"
BUG_REPORT_URL: str = f"{REPO_URL}/issues"
LLM_EXPLORER_URL: str = "https://llm_explorer.streamlit.app/"


@dataclass
class Locale:
    ai_role_options: List[str]
    ai_role_prefix: str
    ai_role_postfix: str
    title: str
    language: str
    lang_code: str
    donates: str
    donates1: str
    donates2: str
    chat_placeholder: str
    chat_run_btn: str
    chat_clear_btn: str
    chat_save_btn: str
    speak_btn: str
    input_kind: str
    input_kind_1: str
    input_kind_2: str
    select_placeholder1: str
    select_placeholder2: str
    select_placeholder3: str
    radio_placeholder: str
    radio_text1: str
    radio_text2: str
    stt_placeholder: str
    footer_title: str
    footer_option0: str
    footer_option1: str
    footer_option2: str
    footer_chat: str
    footer_channel: str
    responsibility_denial: str
    donates_info: str
    tokens_count: str
    message_cost: str
    total_cost: str
    empty_api_handler: str


# --- LOCALE SETTINGS ---
en = Locale(
    ai_role_options=AI_ROLE_OPTIONS_EN,
    ai_role_prefix="You are a female",
    ai_role_postfix="Answer as concisely as possible.",
    title="LLM Explorer",
    language="English",
    lang_code="en",
    donates="Donates",
    donates1="Russia",
    donates2="World",
    chat_placeholder="Start Your Conversation With AI:",
    chat_run_btn="Ask",
    chat_clear_btn="Clear",
    chat_save_btn="Save",
    speak_btn="Push to Speak",
    input_kind="Input Kind",
    input_kind_1="Text",
    input_kind_2="Voice [test mode]",
    select_placeholder1="Select Model",
    select_placeholder2="Select Role",
    select_placeholder3="Create Role",
    radio_placeholder="Role Interaction",
    radio_text1="Select",
    radio_text2="Create",
    stt_placeholder="To Hear The Voice Of AI Press Play",
    footer_title="Support & Feedback",
    footer_option0="Chat",
    footer_option1="Info",
    footer_option2="Donate",
    footer_chat="LLM Explorer Chat",
    footer_channel="LLM Explorer Channel",
    responsibility_denial="""
        `LLM Explorer` uses the `Open AI` API to interact with `ChatGPT`, an AI that generates information.
        Please note that neural network responses may not be reliable, inaccurate or irrelevant.
        We are not responsible for any consequences associated with the use or reliance on the information provided.
        Use the received data at your discretion.
    """,
    donates_info="""
        `LLM Explorer` collects donations solely for the purpose of paying for the `Open AI` API.
        This allows you to provide access to communication with AI for all users.
        Support us for joint development and interaction with the intelligence of the future!
    """,
    tokens_count="Tokens count: ",
    message_cost="Message cost: ",
    total_cost="Total cost of conversation: ",
    empty_api_handler=f"""
        API key not found. Create `.streamlit/secrets.toml` with your API key.
        See [README.md]({README_URL}) for instructions or use the original [LLM Explorer]({LLM_EXPLORER_URL}).
    """,
)


es = Locale(
    ai_role_options=AI_ROLE_OPTIONS_ES,
    ai_role_prefix="Eres una asistente femenina",
    ai_role_postfix="Responde de manera concisa.",
    title="LLM Explorer",
    language="Español",
    lang_code="es",
    donates="Donaciones",
    donates1="Rusia",
    donates2="Mundo",
    chat_placeholder="Comienza tu conversación con AI:",
    chat_run_btn="Preguntar",
    chat_clear_btn="Borrar",
    chat_save_btn="Guardar",
    speak_btn="Presiona para hablar",
    input_kind="Tipo de entrada",
    input_kind_1="Texto",
    input_kind_2="Voz [modo de prueba]",
    select_placeholder1="Seleccionar modelo",
    select_placeholder2="Seleccionar rol",
    select_placeholder3="Crear rol",
    radio_placeholder="Interacción del rol",
    radio_text1="Seleccionar",
    radio_text2="Crear",
    stt_placeholder="Para escuchar la voz de AI, presiona Reproducir",
    footer_title="Soporte y comentarios",
    footer_option0="Chat",
    footer_option1="Información",
    footer_option2="Donar",
    footer_chat="Chat de LLM Explorer",
    footer_channel="Canal de LLM Explorer",
    responsibility_denial="""
        `LLM Explorer` utiliza la API de `Open AI` para interactuar con `ChatGPT`, una IA que genera información.
        Ten en cuenta que las respuestas de la red neuronal pueden no ser confiables, precisas o relevantes.
        No somos responsables de las consecuencias asociadas con el uso o la dependencia de la información proporcionada.
        Utiliza los datos recibidos a tu discreción.
    """,
    donates_info="""
        `LLM Explorer` recopila donaciones exclusivamente con el propósito de pagar por la API de `Open AI`.
        Esto te permite proporcionar acceso a la comunicación con AI para todos los usuarios.
        ¡Apóyanos para el desarrollo conjunto e interacción con la inteligencia del futuro!
    """,
    tokens_count="Cantidad de tokens: ",
    message_cost="Costo por mensaje: ",
    total_cost="Costo total de la conversación: ",
    empty_api_handler="""
        No se encontró la clave de API. Crea un archivo `.streamlit/secrets.toml` con tu clave de API.
        Consulta el [README.md]({README_URL}) para obtener instrucciones o utiliza el [LLM Explorer original]({LLM_EXPLORER_URL}).
    """,
)



def session_state_templates():
    return {
        "greeting": "Welcome to the LLM Explorer 🔍 Please type you prompt in the text box below and hit submit to get started 🚀",
        "default_messages": """Hi, please type your query and I will help you creating: 
        - forecasts
        - job manifests
        - data analysis
        - insights from the lakehouse
        - package usage and functions development
        - and more!
        """,
        "query_form": "Please type you query here",
        "starting_query": "Which are the top 10 producing wells?",
        "prior": [
            "Hi, you are an developer assistant that will use the tools, memory vector stores and chat history to provide the most consistent answer to the user. I will prompt you to help me creating: \n - forecasts \n - job manifests \n - data analysis \n - insights from the lakehouse \n - package usage and functions development \n - and more. Let's think this step by step"
        ],
    }
